"""
_GetPNNtoPSNMapping_

Oracle implementation of Locations.GetPNNtoPSNMapping

"""
from __future__ import (print_function, division)

from WMCore.WMBS.MySQL.Locations.GetPNNtoPSNMapping import GetPNNtoPSNMapping as MySQLGetPNNtoPSNMapping

class GetPNNtoPSNMapping(MySQLGetPNNtoPSNMapping):
    """
    Same as MySQL version

    """
    pass
