"""
_ListSubscriptions_

Oracle implementation of DBS3Buffer.ListSubscriptions

Created on May 2, 2013

@author: dballest
"""

from WMComponent.DBS3Buffer.MySQL.ListSubscriptions import ListSubscriptions as MySQLListSubscriptions

class ListSubscriptions(MySQLListSubscriptions):
    pass
