"""
_GetNumberOfJobsForWorkflowTaskStatus_

Oracle implementation of GetNumberOfJobsForWorkflowTaskStatus

Created on Apr 17, 2013

@author: dballest
"""
__all__ = []

from WMCore.WMBS.MySQL.Jobs.GetNumberOfJobsForWorkflowTaskStatus import GetNumberOfJobsForWorkflowTaskStatus \
    as MySQLGetNumberOfJobsForWorkflowTaskStatus

class GetNumberOfJobsForWorkflowTaskStatus(MySQLGetNumberOfJobsForWorkflowTaskStatus):
    pass
