#!/usr/bin/env python
"""
_UpdateHyperNewsName_

Oracle implementation of Users.UpdateHyperNewsName
"""

from WMCore.WMBS.MySQL.Users.UpdateHyperNewsName import UpdateHyperNewsName as MySQLUpdateHyperNewsName

class UpdateHyperNewsName(MySQLUpdateHyperNewsName):
    pass
