#!/usr/bin/env python
"""
_GetStateID_

Oracle implementation of Jobs.GetStateID

Created on Sep 13, 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Jobs.GetStateID import GetStateID as MySQLGetStateID

class GetStateID(MySQLGetStateID):
    pass
