#!/usr/bin/env python

"""
Oracle implementation of AddChecksum
"""





from WMCore.WMBS.MySQL.Files.AddChecksum import AddChecksum as MySQLAddChecksum

class AddChecksum(MySQLAddChecksum):
    """
    Identical to MySQL Version

    """
