#!/usr/bin/env python
"""
_LoadComplete_

Oracle implementation for loading bl_runjob records for complete jobs
"""


from WMCore.BossAir.MySQL.LoadComplete import LoadComplete as MySQLLoadComplete

class LoadComplete(MySQLLoadComplete):
    """
    _LoadComplete_

    """
